"""Infralens - Infrastructure monitoring dashboard."""

from datetime import datetime

from textual.app import App, ComposeResult
from textual.binding import Binding
from textual.containers import Horizontal
from textual.widgets import (
    DataTable,
    Footer,
    Header,
    Label,
    Static,
    TabbedContent,
    TabPane,
)

from infralens.data import (
    get_domain_alerts,
    get_last_refresh,
    is_data_stale,
    load_ai_costs,
    load_ai_usage,
    load_ai_workspaces,
    load_cloudflare_zones,
    load_domains,
    load_hosting,
)


class SummaryCard(Static):
    DEFAULT_CSS = """
    SummaryCard {
        width: 1fr;
        height: auto;
        border: solid #a65d40;
        padding: 1 2;
        margin: 0 1 0 0;
        background: #2d2d2d;
    }
    SummaryCard:last-of-type {
        margin-right: 0;
    }
    """

    def __init__(self, title: str, value: str, subtitle: str = "") -> None:
        super().__init__()
        self._title = title
        self._value = value
        self._subtitle = subtitle

    def compose(self) -> ComposeResult:
        yield Label(self._title, classes="card-title")
        yield Label(self._value, classes="card-value")
        if self._subtitle:
            yield Label(self._subtitle, classes="card-subtitle")


class InfralensApp(App):
    CSS = """
    * {
        scrollbar-color: #a65d40;
        scrollbar-background: #2d2d2d;
    }

    Screen {
        background: #1e1e1e;
    }

    Header {
        background: #2d2d2d;
        color: #cc7755;
    }

    Footer {
        background: #2d2d2d;
    }

    Footer > .footer-key--key {
        background: #a65d40;
        color: #1e1e1e;
    }

    Footer > .footer-key--description {
        color: #d4d4d4;
    }

    TabbedContent {
        height: 1fr;
    }

    ContentSwitcher {
        height: 1fr;
    }

    TabPane {
        height: 1fr;
        padding: 1;
        background: #1e1e1e;
    }

    Tabs {
        background: #2d2d2d;
    }

    Tab {
        color: #808080;
        background: #2d2d2d;
    }

    Tab:hover {
        color: #d4d4d4;
    }

    Tab.-active {
        color: #cc7755;
        text-style: bold;
        background: #3d3d3d;
    }

    Underline {
        height: 0;
    }

    VerticalScroll {
        height: 1fr;
    }

    .cards-row {
        height: auto;
        margin-bottom: 1;
    }

    .card-title {
        color: #808080;
    }

    .card-value {
        color: #cc7755;
        text-style: bold;
    }

    .card-subtitle {
        color: #808080;
    }

    .section-title {
        color: #cc7755;
        text-style: bold;
        margin: 1 0 0 0;
    }

    .meta-info {
        color: #808080;
        margin: 0 0 1 0;
    }

    .muted {
        color: #808080;
    }

    .success {
        color: #6a9955;
    }

    .alert-critical {
        color: #f14c4c;
    }

    .alert-warning {
        color: #dcdcaa;
    }

    DataTable {
        height: auto;
        max-height: 20;
        margin: 1 0;
        background: #2d2d2d;
    }

    DataTable > .datatable--header {
        background: #2d2d2d;
        color: #cc7755;
        text-style: bold;
    }

    DataTable > .datatable--cursor {
        background: #a65d40;
        color: #d4d4d4;
    }
    """

    BINDINGS = [
        Binding("1", "switch_tab('dashboard')", "1", priority=True),
        Binding("2", "switch_tab('domains')", "2", priority=True),
        Binding("3", "switch_tab('dns')", "3", priority=True),
        Binding("4", "switch_tab('hosting')", "4", priority=True),
        Binding("5", "switch_tab('spend')", "5", priority=True),
        Binding("tab", "next_tab", "Tab", priority=True),
        Binding("shift+tab", "prev_tab", show=False, priority=True),
        Binding("enter", "activate_panel", "Enter", priority=True),
        Binding("escape", "deactivate_panel", "Esc", priority=True),
        Binding("r", "refresh", "Refresh", priority=True),
        Binding("q", "quit", "Quit", priority=True),
    ]

    TITLE = "Cloud Inventory"

    def __init__(self, first_run: bool = False) -> None:
        super().__init__()
        self._panel_active = False
        self._tab_order = ["dashboard", "domains", "dns", "hosting", "spend"]
        self._first_run = first_run

    def compose(self) -> ComposeResult:
        yield Header()
        with TabbedContent():
            with TabPane("[1] Dashboard", id="dashboard"):
                yield from self._dashboard_content()
            with TabPane("[2] Domains", id="domains"):
                yield from self._domains_content()
            with TabPane("[3] DNS", id="dns"):
                yield from self._dns_content()
            with TabPane("[4] Hosting", id="hosting"):
                yield from self._hosting_content()
            with TabPane("[5] Spend", id="spend"):
                yield from self._spend_content()
        yield Footer()

    def _dashboard_content(self) -> ComposeResult:
        domains = load_domains()
        active_domains = [d for d in domains if d.status == "ACTIVE"]
        alerts = get_domain_alerts()
        hosting = load_hosting()
        costs = load_ai_costs()
        last_refresh = get_last_refresh()

        refresh_str = "Never"
        if last_refresh:
            refresh_str = last_refresh.strftime("%Y-%m-%d %H:%M")
            if is_data_stale():
                refresh_str += " (stale)"

        with Horizontal(classes="cards-row"):
            yield SummaryCard("Domains", str(len(active_domains)), "active")
            yield SummaryCard("Alerts", str(len(alerts)), "pending")
            yield SummaryCard("Projects", str(len(hosting)), "hosted")
            yield SummaryCard("AI Spend", f"${costs['total']:.2f}", "this month")

        yield Label(f"Last refresh: {refresh_str}", classes="meta-info")
        yield Label("Alerts", classes="section-title")

        if alerts:
            for level, domain, message in alerts[:10]:
                cls = "alert-critical" if level == "critical" else "alert-warning"
                yield Label(f"  {domain}: {message}", classes=cls)
        else:
            yield Label("  All clear - no alerts", classes="success")

    def _domains_content(self) -> ComposeResult:
        yield Label("Active Domains", classes="section-title")

        table = DataTable()
        table.add_columns("Domain", "Registrar", "Expires", "Auto-Renew")

        now = datetime.now()
        for d in load_domains():
            if d.status != "ACTIVE":
                continue
            exp_str = "-"
            if d.expires:
                days = (d.expires.replace(tzinfo=None) - now).days
                exp_str = f"{d.expires.strftime('%Y-%m-%d')} ({days}d)"
            renew = "Yes" if d.auto_renew else "No"
            table.add_row(d.name, d.registrar, exp_str, renew)

        yield table

    def _dns_content(self) -> ComposeResult:
        yield Label("Cloudflare Zones", classes="section-title")

        table = DataTable()
        table.add_columns("Zone", "Status", "Plan")

        for zone in load_cloudflare_zones():
            if isinstance(zone, dict):
                table.add_row(
                    zone.get("name", ""),
                    zone.get("status", ""),
                    zone.get("plan", {}).get("name", ""),
                )

        yield table

    def _hosting_content(self) -> ComposeResult:
        hosting = load_hosting()
        vercel = [p for p in hosting if p.provider == "Vercel"]
        flyio = [p for p in hosting if p.provider == "Fly.io"]

        yield Label(f"Vercel Projects ({len(vercel)})", classes="section-title")
        if vercel:
            v_table = DataTable()
            v_table.add_columns("Project", "URL")
            for p in vercel:
                v_table.add_row(p.name, p.url or "-")
            yield v_table
        else:
            yield Label("  No projects", classes="muted")

        yield Label(f"Fly.io Apps ({len(flyio)})", classes="section-title")
        if flyio:
            f_table = DataTable()
            f_table.add_columns("App", "Status", "URL")
            for p in flyio:
                f_table.add_row(p.name, p.status, p.url or "-")
            yield f_table
        else:
            yield Label("  No apps", classes="muted")

    def _spend_content(self) -> ComposeResult:
        costs = load_ai_costs()
        usage = load_ai_usage()
        workspaces = load_ai_workspaces()

        yield Label("Cost Summary", classes="section-title")
        yield Label(f"  Total: ${costs['total']:.2f} this month")
        yield Label(f"  OpenAI: ${costs['openai']:.2f} (prepaid)")
        yield Label(f"  Anthropic: ${costs['anthropic']:.2f} (pay-as-you-go)")

        yield Label("Anthropic Usage (7 days)", classes="section-title")
        if usage["anthropic"]:
            a_table = DataTable()
            a_table.add_columns("Model", "Input", "Output", "Total")
            for model, tokens in sorted(usage["anthropic"].items()):
                total = tokens["input"] + tokens["output"]
                a_table.add_row(model, f"{tokens['input']:,}", f"{tokens['output']:,}", f"{total:,}")
            yield a_table
        else:
            yield Label("  No usage", classes="muted")

        yield Label("OpenAI Usage (this month)", classes="section-title")
        if usage["openai"]:
            o_table = DataTable()
            o_table.add_columns("Model", "Input", "Output", "Total")
            for model, tokens in sorted(usage["openai"].items()):
                total = tokens["input"] + tokens["output"]
                o_table.add_row(model, f"{tokens['input']:,}", f"{tokens['output']:,}", f"{total:,}")
            yield o_table
        else:
            yield Label("  No usage this month", classes="muted")

        yield Label("Workspaces & Projects", classes="section-title")
        w_table = DataTable()
        w_table.add_columns("Provider", "Name")
        for w in workspaces:
            w_table.add_row(w.provider, w.name)
        yield w_table

    def on_mount(self) -> None:
        if self._first_run:
            self.notify("Fetching data from providers...")
            self.run_worker(self._do_refresh, exclusive=True, thread=True)

    def action_refresh(self) -> None:
        self.notify("Refreshing data...")
        self.run_worker(self._do_refresh, exclusive=True, thread=True)

    def _do_refresh(self) -> None:
        try:
            from infralens.fetch import fetch_all
            fetch_all()
            self.call_from_thread(self._on_refresh_done)
        except Exception as e:
            self.call_from_thread(lambda: self.notify(f"Refresh failed: {e}", severity="error"))

    def _on_refresh_done(self) -> None:
        self.notify("Data refreshed!", severity="information")
        self.refresh(recompose=True)

    def action_switch_tab(self, tab_id: str) -> None:
        self._panel_active = False
        tabs = self.query_one(TabbedContent)
        tabs.active = tab_id
        tabs.focus()
        self.sub_title = ""

    def action_next_tab(self) -> None:
        if self._panel_active:
            self.screen.focus_next()
        else:
            tabs = self.query_one(TabbedContent)
            current = tabs.active
            if current in self._tab_order:
                idx = self._tab_order.index(current)
                next_idx = (idx + 1) % len(self._tab_order)
                tabs.active = self._tab_order[next_idx]

    def action_prev_tab(self) -> None:
        if self._panel_active:
            self.screen.focus_previous()
        else:
            tabs = self.query_one(TabbedContent)
            current = tabs.active
            if current in self._tab_order:
                idx = self._tab_order.index(current)
                prev_idx = (idx - 1) % len(self._tab_order)
                tabs.active = self._tab_order[prev_idx]

    def action_activate_panel(self) -> None:
        if not self._panel_active:
            self._panel_active = True
            self.sub_title = "[Panel Mode - Tab to navigate, Esc to exit]"
            # Get active tab pane and focus its first table
            tabs = self.query_one(TabbedContent)
            active_pane = tabs.query_one(f"#{tabs.active}", TabPane)
            if active_pane:
                tables = active_pane.query(DataTable)
                if tables:
                    tables.first().focus()

    def action_deactivate_panel(self) -> None:
        if self._panel_active:
            self._panel_active = False
            self.sub_title = ""
            self.set_focus(None)


def main():
    app = InfralensApp()
    app.run()


if __name__ == "__main__":
    main()



"""Fetch data from all cloud providers."""

import os
import json
import urllib.request
import urllib.error
import xml.etree.ElementTree as ET
from datetime import datetime, timedelta
from pathlib import Path

from infralens.config import load_env, DATA_DIR, get_config_dir

# Ensure data directory exists
get_config_dir()

# Also support local data/ for development
_LOCAL_DATA = Path(__file__).parent.parent / "data"
_DATA_DIR = _LOCAL_DATA if _LOCAL_DATA.exists() else DATA_DIR


def save_json(filename: str, data) -> None:
    """Save data to JSON file with timestamp."""
    filepath = _DATA_DIR / filename
    with open(filepath, "w") as f:
        json.dump({
            "fetched_at": datetime.now().isoformat(),
            "data": data
        }, f, indent=2)
    print(f"  Saved: {filename}")


def api_request(url: str, headers: dict, method: str = "GET"):
    """Make API request and return JSON response."""
    req = urllib.request.Request(url, headers=headers, method=method)
    try:
        with urllib.request.urlopen(req, timeout=30) as resp:
            return json.loads(resp.read().decode())
    except urllib.error.HTTPError as e:
        return {"error": e.code, "message": e.read().decode()[:500]}
    except Exception as e:
        return {"error": str(e)}


def fetch_vercel() -> None:
    """Fetch Vercel projects via CLI."""
    print("\n=== Vercel ===")
    import subprocess
    import re
    import shutil

    if not shutil.which("vercel"):
        print("  Vercel CLI not installed")
        return

    all_projects = []
    next_cursor = None

    while True:
        cmd = ["vercel", "project", "ls"]
        if next_cursor:
            cmd.extend(["--next", next_cursor])

        result = subprocess.run(cmd, capture_output=True, text=True)
        output = result.stdout + result.stderr

        for line in output.split("\n"):
            line = line.strip()
            if not line:
                continue
            if any(line.startswith(s) for s in [">", "Vercel", "Fetching", "Project Name"]):
                continue
            parts = line.split()
            if len(parts) >= 2:
                name = parts[0]
                url = parts[1] if parts[1].startswith("http") else None
                if name and not name.startswith("-") and name != "Project":
                    all_projects.append({"name": name, "url": url})

        match = re.search(r'--next (\d+)', output)
        if match:
            next_cursor = match.group(1)
        else:
            break

    save_json("vercel-projects.json", all_projects)
    print(f"  Projects: {len(all_projects)}")


def fetch_flyio() -> None:
    """Fetch Fly.io apps via CLI."""
    print("\n=== Fly.io ===")
    import subprocess
    import shutil

    if not shutil.which("fly") and not shutil.which("flyctl"):
        print("  Fly CLI not installed")
        return

    cmd = "fly" if shutil.which("fly") else "flyctl"
    result = subprocess.run([cmd, "apps", "list", "--json"], capture_output=True, text=True)

    try:
        apps = json.loads(result.stdout)
        save_json("flyio-apps.json", apps)
        print(f"  Apps: {len(apps)}")
    except:
        save_json("flyio-apps.json", [])
        print("  Could not parse apps list")


def fetch_cloudflare() -> None:
    """Fetch Cloudflare zones and registrar domains."""
    print("\n=== Cloudflare ===")
    token = os.environ.get("CLOUDFLARE_API_TOKEN")
    if not token:
        print("  CLOUDFLARE_API_TOKEN not set")
        return

    headers = {"Authorization": f"Bearer {token}", "Content-Type": "application/json"}

    # Get all zones
    all_zones = []
    for page in range(1, 5):
        data = api_request(
            f"https://api.cloudflare.com/client/v4/zones?page={page}&per_page=50",
            headers
        )
        zones = data.get("result", [])
        if not zones:
            break
        all_zones.extend(zones)
    save_json("cloudflare-zones.json", all_zones)
    print(f"  Zones: {len(all_zones)}")

    # Get registrar domains - need to get account ID first
    account_data = api_request("https://api.cloudflare.com/client/v4/accounts", headers)
    accounts = account_data.get("result", [])
    if accounts:
        account_id = accounts[0].get("id")
        data = api_request(
            f"https://api.cloudflare.com/client/v4/accounts/{account_id}/registrar/domains",
            headers
        )
        registrar = data.get("result", [])
        save_json("cloudflare-registrar.json", registrar)
        print(f"  Registrar domains: {len(registrar) if isinstance(registrar, list) else 'error'}")


def fetch_godaddy() -> None:
    """Fetch GoDaddy domains."""
    print("\n=== GoDaddy ===")
    api_key = os.environ.get("GODADDY_API_KEY")
    api_secret = os.environ.get("GODADDY_API_SECRET")
    if not api_key or not api_secret:
        print("  GODADDY credentials not set")
        return

    headers = {
        "Authorization": f"sso-key {api_key}:{api_secret}",
        "Content-Type": "application/json"
    }
    data = api_request("https://api.godaddy.com/v1/domains", headers)
    save_json("godaddy-domains.json", data)
    if isinstance(data, list):
        active = [d for d in data if isinstance(d, dict) and d.get("status") == "ACTIVE"]
        print(f"  Domains: {len(data)} total, {len(active)} active")
    else:
        print(f"  Error fetching domains")


def fetch_namecheap() -> None:
    """Fetch Namecheap domains."""
    print("\n=== Namecheap ===")
    api_key = os.environ.get("NAMECHEAP_API_KEY")
    username = os.environ.get("NAMECHEAP_USERNAME")
    if not api_key or not username:
        print("  NAMECHEAP credentials not set")
        return

    try:
        with urllib.request.urlopen("https://api.ipify.org") as resp:
            client_ip = resp.read().decode()
    except:
        print("  Could not get public IP")
        return

    url = f"https://api.namecheap.com/xml.response?ApiUser={username}&ApiKey={api_key}&UserName={username}&Command=namecheap.domains.getList&ClientIp={client_ip}&PageSize=100"

    try:
        with urllib.request.urlopen(url) as resp:
            xml_data = resp.read().decode()

        root = ET.fromstring(xml_data)
        ns = {'nc': 'http://api.namecheap.com/xml.response'}

        domains = []
        for d in root.findall('.//nc:Domain', ns):
            domains.append({
                "name": d.attrib.get("Name"),
                "expires": d.attrib.get("Expires"),
                "auto_renew": d.attrib.get("AutoRenew") == "true",
                "is_locked": d.attrib.get("IsLocked") == "true"
            })
        save_json("namecheap-domains.json", domains)
        print(f"  Domains: {len(domains)}")
    except Exception as e:
        print(f"  Error: {e}")


def fetch_openai() -> None:
    """Fetch OpenAI projects and usage."""
    print("\n=== OpenAI ===")
    api_key = os.environ.get("OPENAI_ADMIN_KEY")
    if not api_key:
        print("  OPENAI_ADMIN_KEY not set")
        return

    headers = {"Authorization": f"Bearer {api_key}"}

    # Get projects
    data = api_request(
        "https://api.openai.com/v1/organization/projects?limit=100",
        headers
    )
    projects = data.get("data", [])
    save_json("openai-projects.json", projects)
    print(f"  Projects: {len(projects)}")

    # Get usage (this month)
    start_ts = int(datetime.now().replace(day=1, hour=0, minute=0, second=0).timestamp())
    data = api_request(
        f"https://api.openai.com/v1/organization/usage/completions?start_time={start_ts}&limit=30&bucket_width=1d",
        headers
    )
    save_json("openai-usage.json", data)

    # Get costs
    data = api_request(
        f"https://api.openai.com/v1/organization/costs?start_time={start_ts}&limit=30&bucket_width=1d",
        headers
    )
    save_json("openai-costs.json", data)


def fetch_anthropic() -> None:
    """Fetch Anthropic workspaces, API keys, usage, and costs."""
    print("\n=== Anthropic ===")
    api_key = os.environ.get("ANTHROPIC_ADMIN_KEY")
    if not api_key:
        print("  ANTHROPIC_ADMIN_KEY not set")
        return

    headers = {"x-api-key": api_key, "anthropic-version": "2023-06-01"}

    # Get workspaces
    data = api_request("https://api.anthropic.com/v1/organizations/workspaces", headers)
    workspaces = data.get("data", [])
    save_json("anthropic-workspaces.json", workspaces)
    print(f"  Workspaces: {len(workspaces)}")

    # Get API keys
    data = api_request("https://api.anthropic.com/v1/organizations/api_keys?limit=100", headers)
    api_keys = data.get("data", [])
    save_json("anthropic-api-keys.json", api_keys)
    print(f"  API Keys: {len(api_keys)}")

    # Get usage (last 7 days)
    end = datetime.now()
    start = end - timedelta(days=7)
    url = f"https://api.anthropic.com/v1/organizations/usage_report/messages?starting_at={start.strftime('%Y-%m-%dT00:00:00Z')}&ending_at={end.strftime('%Y-%m-%dT23:59:59Z')}&bucket_width=1d&group_by[]=model"
    data = api_request(url, headers)
    save_json("anthropic-usage.json", data)
    print(f"  Usage: {len(data.get('data', []))} days")

    # Get costs (last 30 days)
    start = end - timedelta(days=30)
    url = f"https://api.anthropic.com/v1/organizations/cost_report?starting_at={start.strftime('%Y-%m-%dT00:00:00Z')}&ending_at={end.strftime('%Y-%m-%dT23:59:59Z')}&group_by[]=workspace_id"
    data = api_request(url, headers)
    save_json("anthropic-costs.json", data)

    total_cost = 0
    for bucket in data.get("data", []):
        for r in bucket.get("results", []):
            try:
                total_cost += float(r.get("amount", 0))
            except:
                pass
    print(f"  Cost (30d): ${total_cost:.2f}")


def fetch_all() -> None:
    """Fetch data from all configured providers."""
    print("=" * 50)
    print("Infralens - Fetching Data")
    print(f"Time: {datetime.now().isoformat()}")
    print("=" * 50)

    load_env()

    fetch_vercel()
    fetch_flyio()
    fetch_cloudflare()
    fetch_godaddy()
    fetch_namecheap()
    fetch_openai()
    fetch_anthropic()

    print("\n" + "=" * 50)
    print(f"Data saved to: {_DATA_DIR}")
    print("=" * 50)


if __name__ == "__main__":
    fetch_all()



"""Data loader for cloud inventory JSON files."""

import json
from pathlib import Path
from datetime import datetime
from dataclasses import dataclass

from infralens.config import DATA_DIR as CONFIG_DATA_DIR

# Use local data/ if it exists (development), otherwise use config dir
_LOCAL_DATA = Path(__file__).parent.parent / "data"
DATA_DIR = _LOCAL_DATA if _LOCAL_DATA.exists() else CONFIG_DATA_DIR


@dataclass
class Domain:
    name: str
    registrar: str
    expires: datetime | None
    auto_renew: bool
    status: str


@dataclass
class HostingProject:
    name: str
    provider: str  # vercel or flyio
    url: str | None
    status: str
    last_deploy: datetime | None


@dataclass
class AIWorkspace:
    name: str
    provider: str  # openai or anthropic
    id: str


def load_json(filename: str) -> dict | list:
    path = DATA_DIR / filename
    if not path.exists():
        return {}
    with open(path) as f:
        data = json.load(f)
    return data.get("data", data)


def get_fetch_time(filename: str) -> datetime | None:
    path = DATA_DIR / filename
    if not path.exists():
        return None
    with open(path) as f:
        data = json.load(f)
    if "fetched_at" in data:
        return datetime.fromisoformat(data["fetched_at"])
    return None


def get_last_refresh() -> datetime | None:
    times = []
    for f in DATA_DIR.glob("*.json"):
        t = get_fetch_time(f.name)
        if t:
            times.append(t)
    return min(times) if times else None


def is_data_stale(hours: int = 24) -> bool:
    last = get_last_refresh()
    if not last:
        return True
    return (datetime.now() - last).total_seconds() > hours * 3600


def load_domains() -> list[Domain]:
    domains = []

    # GoDaddy
    for d in load_json("godaddy-domains.json"):
        if isinstance(d, dict):
            exp = None
            if d.get("expires"):
                try:
                    exp = datetime.fromisoformat(d["expires"].replace("Z", "+00:00"))
                except:
                    pass
            domains.append(Domain(
                name=d.get("domain", ""),
                registrar="GoDaddy",
                expires=exp,
                auto_renew=d.get("renewAuto", False),
                status=d.get("status", "UNKNOWN"),
            ))

    # Namecheap
    for d in load_json("namecheap-domains.json"):
        if isinstance(d, dict):
            exp = None
            if d.get("expires"):
                try:
                    exp = datetime.strptime(d["expires"], "%m/%d/%Y")
                except:
                    pass
            domains.append(Domain(
                name=d.get("name", ""),
                registrar="Namecheap",
                expires=exp,
                auto_renew=d.get("auto_renew", False),
                status="ACTIVE",
            ))

    # Cloudflare Registrar
    for d in load_json("cloudflare-registrar.json"):
        if isinstance(d, dict):
            exp = None
            if d.get("expires_at"):
                try:
                    exp = datetime.fromisoformat(d["expires_at"].replace("Z", "+00:00"))
                except:
                    pass
            domains.append(Domain(
                name=d.get("name", ""),
                registrar="Cloudflare",
                expires=exp,
                auto_renew=d.get("auto_renew", False),
                status=d.get("status", "UNKNOWN").upper(),
            ))

    def sort_key(d: Domain) -> datetime:
        if d.expires is None:
            return datetime.max
        # Normalize to naive datetime for comparison
        if d.expires.tzinfo is not None:
            return d.expires.replace(tzinfo=None)
        return d.expires

    return sorted(domains, key=sort_key)


def load_cloudflare_zones() -> list[dict]:
    return load_json("cloudflare-zones.json") or []


def load_hosting() -> list[HostingProject]:
    projects = []

    # Vercel
    vercel_data = load_json("vercel-projects.json")
    if isinstance(vercel_data, list):
        for p in vercel_data:
            projects.append(HostingProject(
                name=p.get("name", ""),
                provider="Vercel",
                url=p.get("url"),
                status="active",
                last_deploy=None,
            ))
    elif isinstance(vercel_data, dict):
        for p in vercel_data.get("projects", []):
            projects.append(HostingProject(
                name=p.get("name", ""),
                provider="Vercel",
                url=p.get("url"),
                status="active",
                last_deploy=None,
            ))

    # Fly.io
    for app in load_json("flyio-apps.json"):
        if isinstance(app, dict):
            projects.append(HostingProject(
                name=app.get("Name", app.get("name", "")),
                provider="Fly.io",
                url=f"https://{app.get('Name', '')}.fly.dev",
                status=app.get("Status", "unknown"),
                last_deploy=None,
            ))

    return projects


def load_ai_workspaces() -> list[AIWorkspace]:
    workspaces = []

    # OpenAI
    for p in load_json("openai-projects.json"):
        if isinstance(p, dict):
            workspaces.append(AIWorkspace(
                name=p.get("name", ""),
                provider="OpenAI",
                id=p.get("id", ""),
            ))

    # Anthropic
    for w in load_json("anthropic-workspaces.json"):
        if isinstance(w, dict):
            workspaces.append(AIWorkspace(
                name=w.get("name", ""),
                provider="Anthropic",
                id=w.get("id", ""),
            ))

    return workspaces


def load_ai_costs() -> dict:
    costs = {"openai": 0.0, "anthropic": 0.0, "total": 0.0}

    # OpenAI costs
    openai_costs = load_json("openai-costs.json")
    if isinstance(openai_costs, dict):
        for bucket in openai_costs.get("data", []):
            for r in bucket.get("results", []):
                try:
                    costs["openai"] += float(r.get("amount", {}).get("value", 0))
                except:
                    pass

    # Anthropic costs
    anthropic_costs = load_json("anthropic-costs.json")
    if isinstance(anthropic_costs, dict):
        for bucket in anthropic_costs.get("data", []):
            for r in bucket.get("results", []):
                try:
                    costs["anthropic"] += float(r.get("amount", 0))
                except:
                    pass

    costs["total"] = costs["openai"] + costs["anthropic"]
    return costs


def load_ai_usage() -> dict:
    """Load usage data aggregated by model."""
    usage = {"anthropic": {}, "openai": {}}

    # Anthropic usage by model
    anthropic_usage = load_json("anthropic-usage.json")
    if isinstance(anthropic_usage, dict):
        for bucket in anthropic_usage.get("data", []):
            for r in bucket.get("results", []):
                model = r.get("model", "unknown")
                input_tokens = r.get("uncached_input_tokens", 0)
                output_tokens = r.get("output_tokens", 0)
                if model not in usage["anthropic"]:
                    usage["anthropic"][model] = {"input": 0, "output": 0}
                usage["anthropic"][model]["input"] += input_tokens
                usage["anthropic"][model]["output"] += output_tokens

    # OpenAI usage by model
    openai_usage = load_json("openai-usage.json")
    if isinstance(openai_usage, dict):
        for bucket in openai_usage.get("data", []):
            for r in bucket.get("results", []):
                model = r.get("model", "unknown")
                input_tokens = r.get("input_tokens", 0)
                output_tokens = r.get("output_tokens", 0)
                if model not in usage["openai"]:
                    usage["openai"][model] = {"input": 0, "output": 0}
                usage["openai"][model]["input"] += input_tokens
                usage["openai"][model]["output"] += output_tokens

    return usage


def get_domain_alerts() -> list[tuple[str, str, str]]:
    """Returns list of (level, domain, message)."""
    alerts = []
    now = datetime.now()

    for d in load_domains():
        if d.status != "ACTIVE":
            continue

        if d.expires:
            days = (d.expires.replace(tzinfo=None) - now).days
            if days < 0:
                alerts.append(("critical", d.name, "EXPIRED"))
            elif days < 30:
                alerts.append(("critical", d.name, f"Expires in {days} days"))
            elif days < 90:
                alerts.append(("warning", d.name, f"Expires in {days} days"))

        if not d.auto_renew and d.status == "ACTIVE":
            alerts.append(("warning", d.name, "Auto-renew disabled"))

    return alerts



"""Interactive setup wizard for Infralens."""

from textual.app import App, ComposeResult
from textual.binding import Binding
from textual.containers import Vertical, Horizontal, VerticalScroll, Center
from textual.widgets import (
    Button,
    Checkbox,
    Footer,
    Header,
    Input,
    Label,
)
from textual.screen import Screen

from infralens.config import PROVIDERS, get_configured_providers, save_env


class WelcomeScreen(Screen):
    """Welcome screen with provider selection."""

    CSS = """
    WelcomeScreen {
        background: #1e1e1e;
        align: center middle;
    }

    .main-container {
        width: 60;
        height: auto;
        max-height: 90%;
        background: #2d2d2d;
        border: solid #3d3d3d;
        padding: 1 2;
    }

    .welcome-title {
        text-align: center;
        color: #cc7755;
        text-style: bold;
        margin-bottom: 1;
    }

    .welcome-subtitle {
        text-align: center;
        color: #808080;
        margin-bottom: 1;
    }

    .provider-item {
        height: 3;
    }

    .provider-status {
        color: #6a9955;
        margin: 0 0 0 2;
    }

    .provider-missing {
        color: #808080;
        margin: 0 0 0 2;
    }

    .button-row {
        margin-top: 1;
        height: 3;
        align: center middle;
    }

    Button {
        margin: 0 1;
    }

    .primary-button {
        background: #a65d40;
    }
    """

    BINDINGS = [
        Binding("enter", "continue", "Continue"),
        Binding("escape", "quit", "Quit"),
        Binding("tab", "focus_next", "Tab", show=False),
        Binding("up", "focus_previous", "Up", show=False),
        Binding("down", "focus_next", "Down", show=False),
    ]

    def __init__(self):
        super().__init__()
        self.selected_providers: set[str] = set()

    def compose(self) -> ComposeResult:
        yield Header()
        configured = get_configured_providers()

        with Vertical(classes="main-container"):
            yield Label("Infralens Setup", classes="welcome-title")
            yield Label("Select providers to configure", classes="welcome-subtitle")

            for provider_id, provider in PROVIDERS.items():
                is_configured = configured.get(provider_id, False)
                with Horizontal(classes="provider-item"):
                    checkbox = Checkbox(provider.name, id=f"check-{provider_id}")
                    if is_configured:
                        checkbox.value = True
                        self.selected_providers.add(provider_id)
                    yield checkbox
                    if is_configured:
                        yield Label("[ok]", classes="provider-status")
                    elif not provider.env_vars:
                        yield Label("[CLI]", classes="provider-missing")

            with Horizontal(classes="button-row"):
                yield Button("Continue", variant="primary", id="continue", classes="primary-button")
                yield Button("Skip", id="skip")
        yield Footer()

    def on_checkbox_changed(self, event: Checkbox.Changed) -> None:
        provider_id = event.checkbox.id.replace("check-", "")
        if event.value:
            self.selected_providers.add(provider_id)
        else:
            self.selected_providers.discard(provider_id)

    def on_button_pressed(self, event: Button.Pressed) -> None:
        if event.button.id == "continue":
            self.action_continue()
        elif event.button.id == "skip":
            self.app.exit()

    def action_continue(self) -> None:
        to_configure = list(self.selected_providers)
        if to_configure:
            self.app.push_screen(ProviderSetupScreen(to_configure))
        else:
            self.app.push_screen(CompleteScreen(False))

    def action_quit(self) -> None:
        self.app.exit()

    def action_focus_next(self) -> None:
        self.screen.focus_next()

    def action_focus_previous(self) -> None:
        self.screen.focus_previous()


class ProviderSetupScreen(Screen):
    """Screen for configuring a single provider."""

    CSS = """
    ProviderSetupScreen {
        background: #1e1e1e;
        align: center middle;
    }

    .setup-container {
        width: 70;
        height: auto;
        padding: 2;
        background: #2d2d2d;
        border: solid #3d3d3d;
    }

    .progress {
        color: #808080;
        text-align: right;
        margin-bottom: 1;
    }

    .provider-title {
        color: #cc7755;
        text-style: bold;
        margin-bottom: 1;
    }

    .security-note {
        color: #6a9955;
        margin-bottom: 1;
    }

    .step-label {
        color: #d4d4d4;
        margin: 0 0 0 2;
    }

    .guide-url {
        color: #cc7755;
        margin: 1 0;
    }

    .input-label {
        color: #808080;
        margin-top: 1;
    }

    .skip-hint {
        color: #808080;
        margin-top: 1;
    }

    Input {
        margin: 0 0 1 0;
        border: tall transparent;
    }

    Input:focus {
        border: tall #a65d40;
    }

    .button-row {
        margin-top: 1;
        height: 3;
        align: center middle;
    }

    Button {
        margin: 0 1;
        border: tall transparent;
    }

    Button:focus {
        border: tall #a65d40;
    }

    Button.-primary {
        background: #a65d40;
    }
    """

    BINDINGS = [
        Binding("escape", "back", "Back"),
        Binding("enter", "next", "Next"),
        Binding("tab", "focus_next", "Tab", show=False),
        Binding("up", "focus_previous", "Up", show=False),
        Binding("down", "focus_next", "Down", show=False),
    ]

    def __init__(self, providers: list[str]):
        super().__init__()
        self.providers = providers
        self.current_index = 0
        self.collected_values: dict[str, str] = {}

    @property
    def current_provider(self):
        return PROVIDERS[self.providers[self.current_index]]

    @property
    def current_provider_id(self):
        return self.providers[self.current_index]

    def compose(self) -> ComposeResult:
        yield Header()
        with Vertical(classes="setup-container"):
            yield Label(
                f"{self.current_index + 1} / {len(self.providers)}",
                classes="progress",
                id="progress"
            )
            yield from self._provider_content()

            with Horizontal(classes="button-row"):
                yield Button("Back", id="back")
                yield Button("Skip", id="skip")
                yield Button("Next", variant="primary", id="next")
        yield Footer()

    def _provider_content(self) -> ComposeResult:
        provider = self.current_provider
        yield Label(f"Configure {provider.name}", classes="provider-title", id="title")

        if provider.env_vars:
            yield Label("Saved locally in ~/.infralens/.env", classes="security-note", id="security")
            yield Label("How to get your key:", classes="input-label", id="steps-label")
        else:
            yield Label("Uses CLI authentication (no API key needed)", classes="security-note", id="security")
            yield Label("Setup steps:", classes="input-label", id="steps-label")

        for i, step in enumerate(provider.guide_steps, 1):
            yield Label(f"  {i}. {step}", classes="step-label", id=f"step-{i}")

        if provider.guide_url:
            yield Label(f"{provider.guide_url}", classes="guide-url", id="url")

        for var in provider.env_vars:
            yield Label(var, classes="input-label", id=f"label-{var}")
            yield Input(
                placeholder=f"Paste {var} here (or leave empty to skip)",
                password="SECRET" in var or "KEY" in var,
                id=f"input-{var}",
            )

        if provider.env_vars:
            yield Label("Leave empty to configure later manually", classes="skip-hint", id="skip-hint")
        else:
            yield Label("Press Next when CLI is installed", classes="skip-hint", id="skip-hint")

    def on_input_submitted(self, event: Input.Submitted) -> None:
        self._save_current()
        self._next_provider()

    def on_button_pressed(self, event: Button.Pressed) -> None:
        if event.button.id == "next":
            self._save_current()
            self._next_provider()
        elif event.button.id == "skip":
            self._next_provider()
        elif event.button.id == "back":
            self.action_back()

    def action_next(self) -> None:
        self._save_current()
        self._next_provider()

    def _save_current(self) -> None:
        provider = self.current_provider
        for var in provider.env_vars:
            try:
                input_widget = self.query_one(f"#input-{var}", Input)
                if input_widget.value:
                    self.collected_values[var] = input_widget.value
            except:
                pass

    def _next_provider(self) -> None:
        self.current_index += 1
        if self.current_index >= len(self.providers):
            if self.collected_values:
                save_env(self.collected_values)
            self.app.push_screen(CompleteScreen(bool(self.collected_values)))
        else:
            self.refresh(recompose=True)

    def action_back(self) -> None:
        if self.current_index > 0:
            self.current_index -= 1
            self.refresh(recompose=True)
        else:
            self.app.pop_screen()

    def action_focus_next(self) -> None:
        self.screen.focus_next()

    def action_focus_previous(self) -> None:
        self.screen.focus_previous()


class CompleteScreen(Screen):
    """Setup complete screen."""

    CSS = """
    CompleteScreen {
        background: #1e1e1e;
        align: center middle;
    }

    .complete-box {
        width: 50;
        height: auto;
        padding: 2;
        background: #2d2d2d;
        border: solid #3d3d3d;
    }

    .complete-title {
        color: #6a9955;
        text-style: bold;
        text-align: center;
        margin-bottom: 1;
    }

    .complete-message {
        color: #d4d4d4;
        text-align: center;
        margin-bottom: 1;
    }

    .complete-path {
        color: #cc7755;
        text-align: center;
        margin-bottom: 1;
    }

    .complete-hint {
        color: #808080;
        text-align: center;
        margin-bottom: 2;
    }

    Button {
        width: 100%;
        border: tall transparent;
    }

    Button:focus {
        border: tall #a65d40;
    }

    Button.-primary {
        background: #a65d40;
    }
    """

    BINDINGS = [
        Binding("enter", "launch", "Launch"),
        Binding("escape", "quit", "Quit"),
    ]

    def __init__(self, has_credentials: bool = True):
        super().__init__()
        self.has_credentials = has_credentials

    def compose(self) -> ComposeResult:
        yield Header()
        with Vertical(classes="complete-box"):
            yield Label("Setup Complete!", classes="complete-title")
            if self.has_credentials:
                yield Label("Credentials saved to:", classes="complete-message")
                yield Label("~/.infralens/.env", classes="complete-path")
            else:
                yield Label("No credentials were added.", classes="complete-message")
                yield Label("Add them manually to ~/.infralens/.env", classes="complete-path")
            yield Label("Run 'infralens fetch' to pull data.", classes="complete-hint")
            yield Button("Launch Infralens", variant="primary", id="launch")
        yield Footer()

    def on_button_pressed(self, event: Button.Pressed) -> None:
        if event.button.id == "launch":
            self.action_launch()

    def action_launch(self) -> None:
        self.app.exit(result="launch")

    def action_quit(self) -> None:
        self.app.exit()


class SetupApp(App):
    """Setup wizard application."""

    CSS = """
    * {
        scrollbar-color: #a65d40;
        scrollbar-background: #2d2d2d;
    }

    Button {
        border: tall transparent;
    }

    Button:focus {
        border: tall #a65d40;
    }

    Button.-primary {
        background: #a65d40;
    }

    Button.-primary:hover {
        background: #cc7755;
    }

    Checkbox {
        border: tall transparent;
    }

    Checkbox:focus {
        border: tall #a65d40;
    }

    Checkbox > .toggle--button {
        background: #3d3d3d;
        color: #808080;
    }

    Checkbox.-on > .toggle--button {
        background: #a65d40;
        color: #1e1e1e;
    }

    Checkbox > .toggle--label {
        text-style: none;
    }

    Checkbox:focus > .toggle--label {
        text-style: bold;
        background: #3d3d3d;
        color: #ffffff;
    }

    Input {
        border: tall transparent;
    }

    Input:focus {
        border: tall #a65d40;
    }
    """

    TITLE = "Infralens Setup"

    def on_mount(self) -> None:
        self.push_screen(WelcomeScreen())

    def on_exit(self, result=None) -> None:
        if result == "launch":
            from infralens.app import InfralensApp
            app = InfralensApp(first_run=True)
            app.run()



"""Infralens CLI entry point."""

import sys

from infralens.config import load_env, ENV_FILE, CONFIG_DIR


def main():
    """Main entry point for infralens command."""
    load_env()

    args = sys.argv[1:]

    if args and args[0] == "setup":
        run_setup()
    elif args and args[0] == "fetch":
        run_fetch()
    else:
        # First run - if no config directory exists, launch setup wizard
        if not CONFIG_DIR.exists() or not ENV_FILE.exists():
            run_setup()
        else:
            # Launch TUI
            from infralens.app import InfralensApp
            app = InfralensApp()
            app.run()


def run_setup():
    """Launch the setup wizard."""
    from infralens.setup import SetupApp
    app = SetupApp()
    app.run()


def run_fetch():
    """Run the data fetcher."""
    # Import here to avoid loading fetch module unless needed
    from infralens import fetch
    fetch.fetch_all()


if __name__ == "__main__":
    main()



"""Configuration management for Infralens."""

import os
from pathlib import Path
from dataclasses import dataclass

CONFIG_DIR = Path.home() / ".infralens"
ENV_FILE = CONFIG_DIR / ".env"
DATA_DIR = CONFIG_DIR / "data"


@dataclass
class Provider:
    name: str
    env_vars: list[str]
    guide_url: str
    guide_steps: list[str]


PROVIDERS = {
    "godaddy": Provider(
        name="GoDaddy",
        env_vars=["GODADDY_API_KEY", "GODADDY_API_SECRET"],
        guide_url="https://developer.godaddy.com/keys",
        guide_steps=[
            "Go to https://developer.godaddy.com/keys",
            "Click 'Create New API Key'",
            "Select 'Production' environment",
            "Copy the Key and Secret",
        ],
    ),
    "cloudflare": Provider(
        name="Cloudflare",
        env_vars=["CLOUDFLARE_API_TOKEN"],
        guide_url="https://dash.cloudflare.com/profile/api-tokens",
        guide_steps=[
            "Go to https://dash.cloudflare.com/profile/api-tokens",
            "Click 'Create Token'",
            "Use 'Read all resources' template (or custom with Zone:Read, DNS:Read)",
            "Copy the token (shown only once)",
        ],
    ),
    "namecheap": Provider(
        name="Namecheap",
        env_vars=["NAMECHEAP_API_KEY", "NAMECHEAP_USERNAME"],
        guide_url="https://ap.www.namecheap.com/settings/tools/apiaccess",
        guide_steps=[
            "Go to https://ap.www.namecheap.com/settings/tools/apiaccess",
            "Enable API Access (may require support ticket)",
            "Add your IP to the whitelist",
            "Copy your API Key and Username",
        ],
    ),
    "vercel": Provider(
        name="Vercel",
        env_vars=[],  # Uses CLI authentication
        guide_url="",
        guide_steps=[
            "Install Vercel CLI: npm i -g vercel",
            "Run: vercel login",
            "No API key needed - uses CLI auth",
        ],
    ),
    "flyio": Provider(
        name="Fly.io",
        env_vars=[],  # Uses CLI authentication
        guide_url="",
        guide_steps=[
            "Install Fly CLI: brew install flyctl (or curl installer)",
            "Run: fly auth login",
            "No API key needed - uses CLI auth",
        ],
    ),
    "openai": Provider(
        name="OpenAI",
        env_vars=["OPENAI_ADMIN_KEY"],
        guide_url="https://platform.openai.com/settings/organization/admin-keys",
        guide_steps=[
            "Go to https://platform.openai.com/settings/organization/admin-keys",
            "Click 'Create new admin key'",
            "Copy the key (starts with sk-admin-)",
            "Note: Requires organization Owner role",
        ],
    ),
    "anthropic": Provider(
        name="Anthropic",
        env_vars=["ANTHROPIC_ADMIN_KEY"],
        guide_url="https://console.anthropic.com/settings/admin-keys",
        guide_steps=[
            "Go to https://console.anthropic.com/settings/admin-keys",
            "Click 'Create Key'",
            "Copy the key (starts with sk-ant-admin-)",
            "Note: Requires organization Admin role",
        ],
    ),
}


def get_config_dir() -> Path:
    """Get or create the config directory."""
    CONFIG_DIR.mkdir(parents=True, exist_ok=True)
    DATA_DIR.mkdir(parents=True, exist_ok=True)
    return CONFIG_DIR


def load_env() -> dict[str, str]:
    """Load environment variables from config file and local .env."""
    env = {}

    # First load from ~/.infralens/.env
    if ENV_FILE.exists():
        env.update(_parse_env_file(ENV_FILE))

    # Then load from local .env (for development)
    local_env = Path.cwd() / ".env"
    if local_env.exists():
        env.update(_parse_env_file(local_env))

    # Apply to os.environ
    for key, value in env.items():
        os.environ.setdefault(key, value)

    return env


def _parse_env_file(path: Path) -> dict[str, str]:
    """Parse a .env file into a dict."""
    env = {}
    with open(path) as f:
        for line in f:
            line = line.strip()
            if not line or line.startswith("#"):
                continue
            if "=" in line:
                key, _, value = line.partition("=")
                env[key.strip()] = value.strip()
    return env


def save_env(values: dict[str, str]) -> None:
    """Save environment variables to config file."""
    get_config_dir()

    # Load existing values
    existing = {}
    if ENV_FILE.exists():
        existing = _parse_env_file(ENV_FILE)

    # Merge with new values
    existing.update(values)

    # Write back with restrictive permissions (owner read/write only)
    with open(ENV_FILE, "w") as f:
        for key, value in sorted(existing.items()):
            f.write(f"{key}={value}\n")

    # Set file permissions to 600 (owner only)
    ENV_FILE.chmod(0o600)


def get_configured_providers() -> dict[str, bool]:
    """Check which providers are configured."""
    load_env()
    result = {}

    for provider_id, provider in PROVIDERS.items():
        if not provider.env_vars:
            # CLI-based providers - check if command exists
            result[provider_id] = _check_cli(provider_id)
        else:
            # API-based providers - check if all env vars are set
            result[provider_id] = all(
                os.environ.get(var) for var in provider.env_vars
            )

    return result


def _check_cli(provider_id: str) -> bool:
    """Check if a CLI tool is available."""
    import shutil

    cli_map = {
        "vercel": "vercel",
        "flyio": "fly",
    }
    cmd = cli_map.get(provider_id)
    return shutil.which(cmd) is not None if cmd else False
